<?PHP
/**
* @copyright Copyright (C) 2008 Zundan. All rights reserved.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
* PhppyaNG! is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
*/


// ----------------------------------------------------------------
//         FUNCIONES para la Pasarela de Pagos
// ----------------------------------------------------------------

// ----------------------------------------------------------
// Function: pasarela_calcularDCC57
//
// Description:
//   Calcula los 2 digitos de control (en base al libro 57) 
//   para la referencia.
//   
// Parameters:
// $enti: codigo de la entidad o emisor
// $sufi: sufijo del emisor
// $refer: referencia (debe tener por lo menos 11 digitos)
// $ident: identificacion de la operacion
// $centeuros: importe en centimos de euros
//   
// Result:
//   Devuelve los 2 digitos de control  
//   
// ----------------------------------------------------------

function pasarela_calcularDCC57 ($enti, $sufi, $refer, $ident, $centeuros){

	// Solo se usan los 11 primeros caracteres.
	$refcutted = substr( $refer, 0, 11);

	// Gehitu emisor + sufijo + referencia (11 karaktere) + identificacion + centeuros
	$suma = $enti + $sufi + $refcutted + $ident + $centeuros;

	// Dividir la suma con 97
	$division = $suma / 97;

	// obtener la posicion del decimal
	$esdecimal = strpos ($division, ".");

	// Si no tiene decimales $esdecimal sera FALSE!
	if (! $esdecimal){
		
		// si no posee decimales, el numero de control a devolver es '00'
		return "00";
	}

	// strstr( $division, ".") obtiene la parte decimal, '.' caracter incluido
	// luego, le quitamos el caracter '.'
	$partedecimal = substr (strstr ($division, "."), 1);

	// Si la parte decimal solo tiene un digito se le agrega un '0' por detras
	if (strlen($partedecimal) == 1){
		$partedecimal = $partedecimal . "0";
	
	} else {
		// si tiene dos o mas decimales, nos quedamos con los primeros 2 decimales
		$partedecimal = substr ($partedecimal, 0, 2);
	}

	$digitocontrol = 100 - $partedecimal;   

	// El digito de control SIEMPRE debe tener 2 digitos!
	if (strlen($digitocontrol) == 1){
		$digitocontrol = "0" . $digitocontrol;
	}
	return ($digitocontrol);
}



// ----------------------------------------------------------
// Function: pasarela_prepararXMLinicial
//
// Description: genera un string conteniendo el XML que
//   se le enviará a la Pasarela de Pagos del Gov. Vasco
//   
// Parameters:
// $formatoaux: formato
// $cpraux: tributo que se va a usar 
// 	(Codigo de Procedimiento Racudatorio) 
// $enti: codigo de la entidad o emisor
// $sufi: sufijo del emisor
// $refer: referencia (debe tener por lo menos 11 digitos)
// $control: los 2 digitos de control
// $ident: identificacion de la operacion
// $importe: importe en centimos de euros
// $tipo: tipo
//   
// Result:
//   Devuelve el string conteniendo el XML  
//   
// ----------------------------------------------------------

function pasarela_prepararXMLinicial ($formatoaux, $cpraux, $enti, $sufi, $refer, $control, $ident, $importe, $tipo, $username, $userdni){

	// Solo se usaran los 11 primeros digitos de la referencia,
	// los otros 2 digitos seran de control. 

	$refcutted = substr ($refer, 0, 11);

	$data = "<?xml version='1.0' encoding='ISO-8859-1'?>\r\n";
	$data .= "<rpcCall module='rpcCoreDaemon'>\r\n";
	$data .= "	<function name='doExecInitializePayment'>\r\n";
	$data .= "		<param name='param0' type='XML'>\r\n";
	$data .= "			<paymentRequestData>\r\n";
	$data .= "				<peticionesPago>\r\n";
	$data .= "					<peticionPago id='0'>\r\n";
	$data .= "						<expediente>\r\n";
	$data .= "							<tercero>\r\n";
	$data .= "								<segundoApellido></segundoApellido>\r\n";
	$data .= "								<primerApellido></primerApellido>\r\n";
	$data .= "								<territorio></territorio>\r\n";
	$data .= "								<tipoTercero>". PASARELAPAGO_CIFNIF . "</tipoTercero>\r\n";
	$data .= "								<razonSocial>".utf8_decode($username)."</razonSocial>\r\n";
	$data .= "								<municipio></municipio>\r\n";
	$data .= "								<codigoPostal></codigoPostal>\r\n";
	$data .= "								<dniNif>".$userdni."</dniNif>\r\n";
	$data .= "								<calle></calle>\r\n";
	$data .= "							</tercero>\r\n";
	$data .= "							<familia></familia>\r\n";
	$data .= "							<codigo></codigo>\r\n";
	$data .= "						</expediente>\r\n";
	$data .= "						<datosPago>\r\n";
	$data .= "							<referencia>" . $refer . $control . "</referencia>\r\n";
	$data .= "							<formato>" . $formatoaux . "</formato>\r\n";
	$data .= "							<cpr>" . $cpraux . "</cpr>\r\n";
	$data .= "							<emisor>" . $enti . "-" . $sufi . "</emisor>\r\n";
	$data .= "							<periodosPago>\r\n";
	$data .= "								<periodoPago id='periodoNormal'>\r\n";
	$data .= "									<identificacion>" . $ident . "</identificacion>\r\n";
	$data .= "									<importe>" . $importe . "</importe>\r\n";
	$data .= "								</periodoPago>\r\n";
	$data .= "							</periodosPago>\r\n";
	$data .= "							<tipo>" . $tipo . "</tipo>\r\n";
	$data .= "						</datosPago>\r\n";
	$data .= "					</peticionPago>\r\n";
	$data .= "				</peticionesPago>\r\n";
	$data .= "			</paymentRequestData>\r\n";
	$data .= "		</param>\r\n";
	$data .= "	</function>\r\n";
	$data .= "</rpcCall>\r\n";

	return ($data);
}



// ----------------------------------------------------------
// Function: pasarela_preparar_datos_redireccion
//
// Description: genera un string conteniendo el HTML,
//   que se usara para redigir al usuario a la Pasarela de Pagos
//   del Gov. Vasco
//
// Parameters:
// $URLredireccion: direccion de la URL a la que se hara la
//   redireccion. Por ejemplo: 
//   http://www.testpago.euskadi.net/p12iWar/p12iRPCDispatcherServlet
// $pagoID: identificador del pago (que se habra obtenido desde la Pasarela)
// $idioma:  idioma con el cual se quiere usa la web de la pasarela
// $modoPago: 
// $IDsesion: 
// $URLretorno: direccion para volver una vez se termine la Pasarela.
//   
// Result:
//   Devuelve los 2 digitos de control  
//   
// ----------------------------------------------------------

function pasarela_preparar_datos_redireccion ($URLredireccion, $pagoID, $idioma, $modoPago, $IDsesion, $URLretorno){

	$data = "<html>\r\n";
	$data .= "<head>\r\n";
	$data .= "<meta http­equiv='Expires'content='Tue, 01 Jan 1980 1:00:00 GMT'>\r\n";
	$data .= "<meta http­equiv='Pragma' content='no­cache'>\r\n";
	$data .= "<meta http­equiv='Content­Type' content='text/html; charset=iso­8859­1' />\r\n";
	$data .= "</head>\r\n";
	$data .= "<body>\r\n";
	$data .= "<form name='p12FormOids' id='p12FormOids' method='POST' action='". $URLredireccion . "'>\r\n";
	$data .= "<input type='hidden' name='module' value='IU'>\r\n";
	$data .= "<input type='hidden' name='p12iOidsPago' value=\"" . $pagoID . ",\">\r\n";
	$data .= "<input type='hidden' name='p12iPresentationRequestData' value=\"<presentationRequestData><idioma>" . $idioma . "</idioma><paymentModes><paymentMode oid='" . $modoPago ."'/><paymentMode oid='02'/></paymentModes></presentationRequestData>\">\r\n";
	$data .= "<input type='hidden' name='p12iProtocolData' value=\"<protocolData><timeStamp>1193382459359</timeStamp><sourceSessionId>" . $IDsesion . "</sourceSessionId><urls><url id='urlVuelta'><![CDATA[" . $URLretorno . "]]></url></urls></protocolData>\">\r\n";
	$data .= "</form>\r\n";
	$data .= "<script language='JavaScript'>document.getElementById('p12FormOids').submit();</script>\r\n";
	$data .= "</body>\r\n";
	$data .= "</html>\r\n";
	
	return ($data);
}



// ----------------------------------------------------------
// Function: pasarela_obten_identificador_pago
//
// Description: una vez que la pasarela (en el paso de la inicializacion)
//   devuelve un XML indicando si la inicializacion  ha sido valida o no.
//   En caso valido, esta funcion obtiene el identificador de Pago que 
//   se incluye dentro del XML 
//
// Parameters:
// $katea: cadena que contiene el XML que la pasarela ha respondido. 
//   
// Result:
//   Devuelve los 2 digitos de control  
//   
// ----------------------------------------------------------
function  pasarela_obten_identificador_pago ($katea)
{

	global $pagoID;

	// Crear anlizador de XML
	$xml_parser = xml_parser_create();

	// La Pasarela de Pago usa la codificacion: ISO-8859-1
	// Aunque por defecto el parser de PHP también lo usa
	// mejor curarse en salud.

	xml_parser_set_option ($xml_parser, XML_OPTION_TARGET_ENCODING, "ISO-8859-1");

	xml_parse_into_struct ($xml_parser, $katea, $valores, $indices);
	
	// Primero habra que ver si existe la etiqueta <validacion>
	// dentro de la respuesta. En caso afirmativo nos indicaria
	// que ha habido un error en el XML que le hemos enviado
	// a la pasarela
	
	if (array_key_exists ('VALIDACION', $indices)) {

		$pagoID = "<b>Error: </b><br/>" . $katea;
		return (FALSE);
	}
	
	// En caso que no aparezca, indica que el XML que se ha enviado
	// a la pasarela es correcta. Por lo que habra que obtener 
	// la identificacion del pago

	if (array_key_exists ('PETICIONPAGO', $indices)) {
		
		foreach ($indices['PETICIONPAGO'] as $key => $valor){
			
			// En la etiqueta de apertura se encuentra el array 'attributes'
			// donde esta la ID del pago que queremos obtener: 
			if ($valores[$valor]['type'] == 'open'){
				
				$pagoID = $valores[$valor]['attributes']['ID'];
				return (TRUE);
			}		
		}
		
	} else {
	   	$pagoID = "Error:  El elemento 'PETICIONPAGO' no se encuentra en la respuesta<br>/";
   		return (FALSE);	
	}
	
}


?>

